﻿using System.Collections.Generic;
using System.Linq;

namespace Workshops.Model.Repositories
{
    public class WorkshopsRepository : IWorkshopsRepository
    {
        private readonly WorkshopsDatabaseContext _context;

        public WorkshopsRepository(WorkshopsDatabaseContext context)
        {
            _context = context;
        }

        public IEnumerable<Workshop> GetAllWorkshops()
        {
            var workshopEntities = _context.WorkshopEntities
                                    .Include("Attendees")
                                    .OrderByDescending(w => w.EventDate)
                                    .ToList();

            // mapping
            // TODO: evaluate if this could be done by a framework (AutoMapper etc.)
            var workshops = new List<Workshop>();
            foreach (var workshopEntity in workshopEntities)
            {
                // map attendees
                var attendees = new List<Attendee>();
                foreach (var attendeeEntity in workshopEntity.Attendees)
                {
                    var attendee = new Attendee
                                       {
                                           Id = attendeeEntity.Id,
                                           FirstName = attendeeEntity.FirstName,
                                           LastName = attendeeEntity.LastName
                                       };
                    attendees.Add(attendee);
                }

                // map workshop
                var workshop = new Workshop
                                   {
                                       Id = workshopEntity.Id,
                                       Title = workshopEntity.Title,
                                       Description = workshopEntity.Description,
                                       EventDate = workshopEntity.EventDate,
                                       Attendees = attendees
                                   };
                
                workshops.Add(workshop);
            }

            return workshops;
        }

        public void AddWorkshop(Workshop workshop)
        {
            var workshopEntity = new WorkshopEntity
                                     {
                                         Title = workshop.Title,
                                         Description = workshop.Description,
                                         EventDate = workshop.EventDate
                                     };

            foreach (var attendee in workshop.Attendees)
            {
                AddNewAttendee(attendee, workshopEntity);
            }

            _context.WorkshopEntities.Add(workshopEntity);
        }

        private static void AddNewAttendee(Attendee attendee, WorkshopEntity workshopEntity)
        {
            var attendeeEntity = new AttendeeEntity
                                     {
                                         FirstName = attendee.FirstName,
                                         LastName = attendee.LastName
                                     };

            workshopEntity.Attendees.Add(attendeeEntity);
        }

        public void EditWorkshop(Workshop workshop)
        {
            var workshopEntity = _context.WorkshopEntities
                                         .Include("Attendees")
                                         .Where(w => w.Id == workshop.Id)
                                         .Single();

            foreach (var attendee in workshop.Attendees)
            {
                if (attendee.Id == 0)
                {
                    // new attendee
                    AddNewAttendee(attendee, workshopEntity);
                }
                else
                {
                    // map back changes
                    var attendeeEntity = workshopEntity.Attendees
                                                       .Where(a => a.Id == attendee.Id)
                                                       .Single();

                    attendeeEntity.FirstName = attendee.FirstName;
                    attendeeEntity.LastName = attendee.LastName;
                }
            }

            // make sure we catch the deleted ones as well
            foreach (var attendeeEntity in workshopEntity.Attendees.ToList())
            {
                var attendee = workshop.Attendees
                                       .Where(a => a.Id == attendeeEntity.Id)
                                       .SingleOrDefault();

                if (attendee == null)
                {
                    workshopEntity.Attendees.Remove(attendeeEntity);
                }
            }
        }
    }
}
